function volume=mixture_volume(A,b,Aeq,beq,num_factors)
%Calculates the volume of the experimental region of a mixture experiment.
%First, the volume of the region only defined by upper and lower bound
%constraints is calculated using the formula of Crosier. Then a random
%sample from this region is used to estimate the volume of the experimental
%region.
Svol=sqrt(num_factors)/factorial(num_factors-1);
R=b(2:2:2*num_factors)+b(1:2:2*num_factors-1);
RL=1-sum(-b(1:2:2*num_factors-1));
RU=sum(b(2:2:2*num_factors))-1;
RP=min(RL,RU);
B=R'./RP;
H=zeros(num_factors,1);
indices=dec2bin(1:2^num_factors-1,num_factors);
ind_bin=logical(indices-'0');
for i=1:num_factors %number of subscripts of H_ijk etc 
   test=(sum(logical(ind_bin),2)==i);
   sets=ind_bin(test,:);
   B2=repmat(B,nchoosek(num_factors,i),1);
   sets=B2.*sets;
   T=zeros(2,nchoosek(num_factors,i));
   T(1,:)=(1-sum(sets,2))';
   T=max(T);
   T=T.^(num_factors-1);
   H(i)=sum(T);
   
end
if mod(num_factors,2)==1
   H(1:2:num_factors)=-1.*H(1:2:num_factors);
else
   H(1:2:num_factors-1)=-1.*H(1:2:num_factors-1);
end
ULvolume=RP^(num_factors-1)*Svol*(1-sum(H));


%ULvolume is the volume of the region defined only by the upper and lower
%bound constraints. Now sample from this region (since its volume is known
%exactly) to estimate the volume of the feasible region including the
%multicomponent constraints.

N=1000;
sample=rejection_sampling(N,num_factors,A(1:2*num_factors,:),b(1:2*num_factors));
sample=sample';
As=A*sample;
rb=repmat(b,1,N);
test=(As<=rb);
test=sum(test);
feasible=sum(test==length(b));

volume=ULvolume*feasible/N;


end
