function [models,repCount]=select_models(numFactors,n2fi,n3fi,n4fi,ndif,numModels)
%Randomly selects numModels many models for an experiment with numFactors many
%components.
%Generates an array of models with a specified number n2fi of second order
%blending terms xixj and n3fi third order blending terms xixjxk. Also
%creates rows for difference terms of the form x1^2*x2-x1*x2^2. Two rows
%are created for each difference term with ndif being the number of
%difference terms to create. These difference terms are the last rows of
%the model matrix.

repCount=0; %counts number of times algorithm generates a model already generated. This is only for performance testing purposes.
nAll2fi=nchoosek(numFactors,2); %total number of 2fi terms
nAll3fi=nchoosek(numFactors,3); %total number of 3fi terms
if numFactors>=4
nAll4fi=nchoosek(numFactors,4);
All4fis=nchoosek(1:numFactors,4);
else
    nAll4fi=0;
    All4fis=0;
end
All2fis=nchoosek(1:numFactors,2); %nchoosek(numFactors,2) by 2 matrix with each row one of the combinations of 2 factors
All3fis=nchoosek(1:numFactors,3); %similarly for 3 factors



models=zeros(numFactors+n2fi+n3fi+n4fi+2*ndif,numFactors,numModels); %each model will include all main effects, n2fi second order blending, n3fi third order blending, and ndif difference terms
strmodels=cell(1,numModels); 
strmodels(:)={char(zeros(numFactors+n2fi+n3fi+n4fi+2*ndif,numFactors))}; %create a cell array that will hold the models generated as char format for quickly testing if a new model had previously been generated

j=1;
while j<=numModels

l2fi=zeros(nAll2fi,1); %these vectors are made to index the choices of terms from ALL2fis/All3fis
l3fi=zeros(nAll3fi,1);
ldif=zeros(nAll2fi,1);
l4fi=zeros(nAll4fi,1);

l2fi(randperm(nAll2fi,n2fi))=1; %randomly choose n2fi terms of l2fi to be one which will select the appropriate rows of All2fis 
l3fi(randperm(nAll3fi,n3fi))=1;
ldif(randperm(nAll2fi,ndif))=1; %difference terms can be randomly generated from by choosing two elements same as for the n2fi terms
l4fi(randperm(nAll4fi,n4fi))=1;

sub2fi=(All2fis(l2fi==1,:)); %subset of the rows of All2fis associated to the ones of l2fi
                             %Each row of sub2fi specifies 2 factors that
                             %will form a parameter xixj in the generated
                             %model.
sub3fi=(All3fis(l3fi==1,:));

subdif=(All2fis(ldif==1,:));

sub4fi=(All4fis(l4fi==1,:));

modelj=zeros(numFactors+n2fi+n3fi+n4fi+2*ndif,numFactors);
modelj(1:numFactors,:)=eye(numFactors); %always include the main effects in every model

for i=1:n2fi
    modelj(numFactors+i,sub2fi(i,:))=1; %convert the sub2fi rows to form necessary for x2fx
end
for k=1:n3fi
   modelj(numFactors+n2fi+k,sub3fi(k,:))=1;
end
for k=1:n4fi
    modelj(numFactors+n2fi+n3fi+k,sub4fi(k,:))=1;
end
for i=1:2:2*ndif
   modelj(numFactors+n2fi+n3fi+n4fi+i,subdif((i+1)/2,1))=2; %Two rows must be created for each difference. 
                                                       %The first element is squared in the first row and the second one is squared in the second row.
   modelj(numFactors+n2fi+n3fi+n4fi+i,subdif((i+1)/2,2))=1;
   modelj(numFactors+n2fi+n3fi+n4fi+i+1,subdif((i+1)/2,1))=1;
   modelj(numFactors+n2fi+n3fi+n4fi+i+1,subdif((i+1)/2,2))=2;
end

strModelj={char(modelj)}; %convert modelj to char to test if it has been generated before in the loop
if ~any(strcmp(strModelj,strmodels))
    models(:,:,j)=modelj; %if modelj is a new model, add it to models
    strmodels{j}=char(strModelj); %keep the char of modelj to test against the later models generated
    j=j+1; %increase the index if a new model was made in this loop
else
    repCount=repCount+1; %if modelj was already created, do not increase the index j and increase the count of repeated models by 1
end

end
end