% script to read in designs from Table 3 and evaluate them in terms of the
% set of models induced by assumed model space

% for QE Revision 1

% Note that subsequent runs of this code may not produce exactly the same
% numbers as in the paper, because of the random selection of models (S_2)
% will be different

% all the examples here are based on examples mentioned in Table 1 of our
% paper; so when we refer to Examples 1, 2, etc., it is with reference to
% this table

% current directory needs to be the directory that is parent to 
% the 'Code_evaluation' directory
curDir = cd;
examplesFolder = fullfile(curDir,'Code_evaluation','examples_paper');

%%% First part here just obtaining information about each example from
%%% Table 1

% need to get the A and b matrices for each example from Table 1
E={'goldfarb2004 3 factor.mat';
    'cornell1990 4 factor p186.mat';
    'heredia_langner2004 5 factor.mat';
    'piepel1983 6 component.mat';
    'smith2005 6 factor p233.mat';
    'piepel1982 11 component.mat';
    'fleury2014 12 factor.mat';
    'piepel 21 component.mat'};

As=cell(length(E),1);
Aeqs=cell(length(E),1);
bs=cell(length(E),1);
beqs=cell(length(E),1);

% gets constraints from the examples
for i=1:length(E)
    load(char(fullfile(examplesFolder,E(i))));
    As{i}=A;
    Aeqs{i}=Aeq;
    bs{i}=b;
    beqs{i}=beq;
end

%%% End of part to get the information for each example

%%% Beginning of part that gets particular designs

% type of design (D (as in D-optimal for special cubic), I, Imr (as in
%  D-model-robust), Imr, Bayes), 
% which example (Table 1 of the paper), 
% k, n, 
% num2fis, num3fis (included even for special cubic optimal designs, though
%   they aren't directly used for the design)
% potential terms prior variance (in filename, taking prior times
%   1000 to make sure it is an integer; for non-Bayes designs, this is 0)
% Number of models randomly taken from model space (1 if D/I-opt or Bayes)

% Table 2
C2={'Imr',2,4,10,3,0,0,10;
    'Imr',2,4,10,3,0,0,16;
    'Imr',2,4,10,3,0,0,20;
    'BayesI',2,4,10,3,0,.001,1;
    'Imr',2,4,10,3,1,0,16;
    'Imr',2,4,10,3,1,0,32;
    'Imr',2,4,10,3,1,0,64;
    'BayesI',2,4,10,3,1,.001,1;
    'Imr',3,5,14,4,0,0,16;
    'Imr',3,5,14,4,0,0,32;
    'Imr',3,5,14,4,0,0,64;
    'BayesI',3,5,14,4,0,.001,1;
    'Imr',3,5,20,4,3,0,16;
    'Imr',3,5,20,4,3,0,32;
    'Imr',3,5,20,4,3,0,64;
    'Imr',3,5,20,4,3,0,128;
    'BayesI',3,5,20,4,3,.001,1;
    'Imr',4,6,20,7,0,0,16;
    'Imr',4,6,20,7,0,0,32;
    'Imr',4,6,20,7,0,0,64;
    'BayesI',4,6,20,7,0,.001,1;
    'Imr',4,6,30,7,7,0,16;
    'Imr',4,6,30,7,7,0,32;
    'Imr',4,6,30,7,7,0,64;
    'Imr',4,6,30,7,7,0,128;
    'BayesI',4,6,30,7,7,.001,1};    

[num_scenarios]=size(C2);
num_scenarios=num_scenarios(1);

% Read in the files specified by C2
M=cell(num_scenarios,1); % designs
avgDs=zeros(num_scenarios,1); % 
avgDs_SC=zeros(num_scenarios,1); % for special cubic
avgPVs=zeros(num_scenarios,1); % 
avgPVs_SC=zeros(num_scenarios,1); % 
Ieffs=cell(num_scenarios,1);
for i=1:num_scenarios
    opt_type=C2{i,1};
    example=C2{i,2};
    k=C2{i,3};
    n=C2{i,4};
    num2fis=C2{i,5};
    num3fis=C2{i,6};
    prior=C2{i,7};
    r=C2{i,8};
    % generate set of models
    % if model space has 10000 or less, take the full model space,
    % otherwise take a random sample of 10000.
    numModels=nchoosek(nchoosek(k,2),num2fis)*nchoosek(nchoosek(k,3),num3fis);

    cd Code_evaluation
    
    if numModels <= 10000
        models=FullModelSpace(k,num2fis,num3fis,0);
    else
        models=select_models(k,num2fis,num3fis,0,0,10000);
    end
     
    cd ..
    cd Designs/current-QE-R1/Table3
    
    if any(strcmp(opt_type,{'BayesD','BayesI'}))
        matFileName = sprintf('%s-ex%d-k%d-n%d-2fi%d-3fi%d-prior%d.csv', opt_type, example, k, n, num2fis,num3fis,prior*1000);
        M{i} = csvread(matFileName,1,0);
    elseif any(strcmp(opt_type,{'D','I'}))
        matFileName = sprintf('%s-optimal-ex%d-k%d-n%d-2fi%d-3fi%d.csv', opt_type, example, k, n, num2fis,num3fis);
        M{i} = csvread(matFileName,1,0);
    elseif any(strcmp(opt_type,{'Dmr'}))
        matFileName = sprintf('%s-ex%d-k%d-Runs_%d n2fi_%d n3fi_%d n4fi_0 ndif_0 numModels_%d.mat',opt_type,example, k, n, num2fis, num3fis,r);
        temp = struct2cell(load(matFileName,'design'));
        M{i}=temp{1,1};
    elseif any(strcmp(opt_type,{'Imr'}))
        matFileName = sprintf('%s-ex%d-k%d-Runs_%d n2fi_%d n3fi_%d n4fi_0 ndif_0 numModels_%d tol_1e-12.mat',opt_type,example, k, n, num2fis, num3fis,r);
        temp = struct2cell(load(matFileName,'design'));
        M{i}=temp{1,1};
    end
    
    A=As{example};
    b=bs{example};
    Aeq=Aeqs{example};
    beq=beqs{example};
    
    cd ..
    cd ..
    cd ..
    cd Code_evaluation
    
    % average D-efficiency
    %[avgDs(i),Deffs]=average_efficiency(M{i},models);
    % average I-efficiency
    [avgPVs(i),Ieffs{i}]=mean_avp(M{i},models,A,b,Aeq,beq,k);
    %[avgPVs_SC(i),Ieffs_SC]=mean_avp(M{i},modelSC,A,b,Aeq,beq,k);
    
    cd ..
end

% if any(strcmp(opt_type,{'BayesD','BayesI'}))
%     name = sprintf('%s-ex%d-k%d-n%d-2fi%d-3fi%d-prior%d_results', opt_type, example, k, n, num2fis,num3fis,prior*1000);
% elseif any(strcmp(opt_type,{'D','I'}))
%     name = sprintf('%s-ex%d-k%d-n%d-2fi%d-3fi%d_results', opt_type, example, k, n, num2fis,num3fis,prior*1000);
% elseif any(strcmp(opt_type,{'Dmr','Imr'}))
%     name = sprintf('%s-ex%d-k%d-n%d-2fi%d-3fi%d-NumModels%d_results', opt_type, example, k, n, num2fis,num3fis,r);
% end

name = sprintf('Table3-results')

% save all variables currently in the Matlab environment
%   * filename will be something.mat
%   * use the command 'load something' to bring them back in
%   * if you don't want any previously existing variables saved
%        as well, use 'clear all' before calling this script
save(name)    







